@@ -12,13 +12,19 @@
 //
 // Copyright (C) 2005 Martin Kretzschmar <martink@gnome.org>
 // Copyright (C) 2005, 2006 Kristian Høgsberg <krh@redhat.com>
-// Copyright (C) 2005, 2007, 2008 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2005, 2007-2010 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2005 Jonathan Blandford <jrb@redhat.com>
 // Copyright (C) 2006, 2007 Jeff Muizelaar <jeff@infidigm.net>
 // Copyright (C) 2006 Takashi Iwai <tiwai@suse.de>
 // Copyright (C) 2006 Ed Catmur <ed@catmur.co.uk>
 // Copyright (C) 2007 Krzysztof Kowalczyk <kkowalczyk@gmail.com>
-// Copyright (C) 2007 Jonathan Kew <jonathan_kew@sil.org>
+// Copyright (C) 2007, 2009 Jonathan Kew <jonathan_kew@sil.org>
+// Copyright (C) 2009 Petr Gajdos <pgajdos@novell.com>
+// Copyright (C) 2009 William Bader <williambader@hotmail.com>
+// Copyright (C) 2009 Kovid Goyal <kovid@kovidgoyal.net>
+// Copyright (C) 2010 Hib Eris <hib@hiberis.nl>
+// Copyright (C) 2010 Patrick Spendrin <ps_ml@gmx.de>
+// Copyright (C) 2010 Jakub Wilk <ubanus@users.sf.net>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -35,12 +41,13 @@
 #include <stdio.h>
 #include <ctype.h>
 #ifdef ENABLE_PLUGINS
-#  ifndef WIN32
+#  ifndef _WIN32
 #    include <dlfcn.h>
 #  endif
 #endif
-#ifdef WIN32
+#ifdef _WIN32
 #  include <shlobj.h>
+#  include <mbstring.h>
 #endif
 #include "goo/gmem.h"
 #include "goo/GooString.h"
@@ -60,8 +67,10 @@
 #include "GlobalParams.h"
 #include "GfxFont.h"
 
-#ifdef WIN32
+#ifdef _WIN32
 #  define strcasecmp stricmp
+#else
+#  include <strings.h>
 #endif
 
 #if MULTITHREADED
@@ -89,7 +98,7 @@
 #include "UTF8.h"
 
 #ifdef ENABLE_PLUGINS
-#  ifdef WIN32
+#  ifdef _WIN32
 extern XpdfPluginVecTable xpdfPluginVecTable;
 #  endif
 #endif
@@ -137,7 +146,63 @@ DisplayFontParam::~DisplayFontParam() {
   }
 }
 
-#ifdef WIN32
+#if ENABLE_RELOCATABLE && defined(_WIN32)
+
+/* search for data relative to where we are installed */
+
+static HMODULE hmodule;
+
+extern "C" {
+BOOL WINAPI
+DllMain (HINSTANCE hinstDLL,
+	 DWORD     fdwReason,
+	 LPVOID    lpvReserved)
+{
+  switch (fdwReason)
+    {
+    case DLL_PROCESS_ATTACH:
+      hmodule = hinstDLL;
+      break;
+    }
+
+  return TRUE;
+}
+}
+
+static char *
+get_poppler_datadir (void)
+{
+  static char retval[MAX_PATH];
+  static int beenhere = 0;
+
+  unsigned char *p;
+
+  if (beenhere)
+    return retval;
+
+  if (!GetModuleFileName (hmodule, (CHAR *) retval, sizeof(retval) - 20))
+    return POPPLER_DATADIR;
+
+  p = _mbsrchr ((unsigned char *) retval, '\\');
+  *p = '\0';
+  p = _mbsrchr ((unsigned char *) retval, '\\');
+  if (p) {
+    if (stricmp ((const char *) (p+1), "bin") == 0)
+      *p = '\0';
+  }
+  strcat (retval, "\\share\\poppler");
+
+  beenhere = 1;
+
+  return retval;
+}
+
+#undef POPPLER_DATADIR
+#define POPPLER_DATADIR get_poppler_datadir ()
+
+#endif
+
+#ifdef _WIN32
 
 //------------------------------------------------------------------------
 // WinFontInfo
@@ -380,7 +445,7 @@ int CALLBACK WinFontList::enumFunc2(CONST LOGFONT *font,
   return 1;
 }
 
-#endif // WIN32
+#endif // _WIN32
 
 //------------------------------------------------------------------------
 // PSFontParam
@@ -415,7 +480,7 @@ public:
 
 private:
 
-#ifdef WIN32
+#ifdef _WIN32
   Plugin(HMODULE libA);
   HMODULE lib;
 #else
@@ -429,7 +494,7 @@ Plugin *Plugin::load(char *type, char *name) {
   Plugin *plugin;
   XpdfPluginVecTable *vt;
   XpdfBool (*xpdfInitPlugin)(void);
-#ifdef WIN32
+#ifdef _WIN32
   HMODULE libA;
 #else
   void *dlA;
@@ -440,7 +505,7 @@ Plugin *Plugin::load(char *type, char *name) {
   appendToPath(path, type);
   appendToPath(path, name);
 
-#ifdef WIN32
+#ifdef _WIN32
   path->append(".dll");
   if (!(libA = LoadLibrary(path->getCString()))) {
     error(-1, "Failed to load plugin '%s'",
@@ -474,7 +539,7 @@ Plugin *Plugin::load(char *type, char *name) {
   }
   memcpy(vt, &xpdfPluginVecTable, sizeof(xpdfPluginVecTable));
 
-#ifdef WIN32
+#ifdef _WIN32
   if (!(xpdfInitPlugin = (XpdfBool (*)(void))
 	                     GetProcAddress(libA, "xpdfInitPlugin"))) {
     error(-1, "Failed to find xpdfInitPlugin in plugin '%s'",
@@ -495,7 +560,7 @@ Plugin *Plugin::load(char *type, char *name) {
     goto err2;
   }
 
-#ifdef WIN32
+#ifdef _WIN32
   plugin = new Plugin(libA);
 #else
   plugin = new Plugin(dlA);
@@ -505,7 +570,7 @@ Plugin *Plugin::load(char *type, char *name) {
   return plugin;
 
  err2:
-#ifdef WIN32
+#ifdef _WIN32
   FreeLibrary(libA);
 #else
   dlclose(dlA);
@@ -515,7 +580,7 @@ Plugin *Plugin::load(char *type, char *name) {
   return NULL;
 }
 
-#ifdef WIN32
+#ifdef _WIN32
 Plugin::Plugin(HMODULE libA) {
   lib = libA;
 }
@@ -528,7 +593,7 @@ Plugin::Plugin(void *dlA) {
 Plugin::~Plugin() {
   void (*xpdfFreePlugin)(void);
 
-#ifdef WIN32
+#ifdef _WIN32
   if ((xpdfFreePlugin = (void (*)(void))
                             GetProcAddress(lib, "xpdfFreePlugin"))) {
     (*xpdfFreePlugin)();
@@ -548,15 +613,12 @@ Plugin::~Plugin() {
 // parsing
 //------------------------------------------------------------------------
 
-GlobalParams::GlobalParams() {
+GlobalParams::GlobalParams(const char *customPopplerDataDir)
+  : popplerDataDir(customPopplerDataDir)
+{
   UnicodeMap *map;
   int i;
 
-#ifndef _MSC_VER  
-  FcInit();
-  FCcfg = FcConfigGetCurrent();
-#endif
-
 #if MULTITHREADED
   gInitMutex(&mutex);
   gInitMutex(&unicodeMapCacheMutex);
@@ -574,7 +636,7 @@ GlobalParams::GlobalParams() {
     }
   }
 
-#ifdef WIN32
+#ifdef _WIN32
   // baseDir will be set by a call to setBaseDir
   baseDir = new GooString();
 #else
@@ -599,11 +661,12 @@ GlobalParams::GlobalParams() {
   psEmbedTrueType = gTrue;
   psEmbedCIDPostScript = gTrue;
   psEmbedCIDTrueType = gTrue;
+  psSubstFonts = gTrue;
   psPreload = gFalse;
   psOPI = gFalse;
   psASCIIHex = gFalse;
   textEncoding = new GooString("UTF-8");
-#if defined(WIN32)
+#if defined(_WIN32)
   textEOL = eolDOS;
 #elif defined(MACOS)
   textEOL = eolMac;
@@ -635,7 +698,8 @@ GlobalParams::GlobalParams() {
   unicodeMapCache = new UnicodeMapCache();
   cMapCache = new CMapCache();
 
-#ifdef WIN32
+#ifdef _WIN32
+  baseFontsInitialized = gFalse;
   winFontList = NULL;
 #endif
 
@@ -673,8 +737,14 @@ GlobalParams::GlobalParams() {
 void GlobalParams::scanEncodingDirs() {
   GDir *dir;
   GDirEntry *entry;
-
-  dir = new GDir(POPPLER_DATADIR "/nameToUnicode", gTrue);
+  const char *dataRoot = popplerDataDir ? popplerDataDir : POPPLER_DATADIR;
+  
+  // allocate buffer large enough to append "/nameToUnicode"
+  size_t bufSize = strlen(dataRoot) + strlen("/nameToUnicode") + 1;
+  char *dataPathBuffer = new char[bufSize];
+  
+  snprintf(dataPathBuffer, bufSize, "%s/nameToUnicode", dataRoot);
+  dir = new GDir(dataPathBuffer, gTrue);
   while (entry = dir->getNextEntry(), entry != NULL) {
     if (!entry->isDir()) {
       parseNameToUnicode(entry->getFullPath());
@@ -683,27 +753,32 @@ void GlobalParams::scanEncodingDirs() {
   }
   delete dir;
 
-  dir = new GDir(POPPLER_DATADIR "/cidToUnicode", gFalse);
+  snprintf(dataPathBuffer, bufSize, "%s/cidToUnicode", dataRoot);
+  dir = new GDir(dataPathBuffer, gFalse);
   while (entry = dir->getNextEntry(), entry != NULL) {
     addCIDToUnicode(entry->getName(), entry->getFullPath());
     delete entry;
   }
   delete dir;
 
-  dir = new GDir(POPPLER_DATADIR "/unicodeMap", gFalse);
+  snprintf(dataPathBuffer, bufSize, "%s/unicodeMap", dataRoot);
+  dir = new GDir(dataPathBuffer, gFalse);
   while (entry = dir->getNextEntry(), entry != NULL) {
     addUnicodeMap(entry->getName(), entry->getFullPath());
     delete entry;
   }
   delete dir;
 
-  dir = new GDir(POPPLER_DATADIR "/cMap", gFalse);
+  snprintf(dataPathBuffer, bufSize, "%s/cMap", dataRoot);
+  dir = new GDir(dataPathBuffer, gFalse);
   while (entry = dir->getNextEntry(), entry != NULL) {
     addCMapDir(entry->getName(), entry->getFullPath());
     toUnicodeDirs->append(entry->getFullPath()->copy());
     delete entry;
   }
   delete dir;
+  
+  delete[] dataPathBuffer;
 }
 
 void GlobalParams::parseNameToUnicode(GooString *name) {
@@ -712,6 +787,7 @@ void GlobalParams::parseNameToUnicode(GooString *name) {
   char buf[256];
   int line;
   Unicode u;
+  char *tokptr;
 
   if (!(f = fopen(name->getCString(), "r"))) {
     error(-1, "Couldn't open 'nameToUnicode' file '%s'",
@@ -720,8 +796,8 @@ void GlobalParams::parseNameToUnicode(GooString *name) {
   }
   line = 1;
   while (getLine(buf, sizeof(buf), f)) {
-    tok1 = strtok(buf, " \t\r\n");
-    tok2 = strtok(NULL, " \t\r\n");
+    tok1 = strtok_r(buf, " \t\r\n", &tokptr);
+    tok2 = strtok_r(NULL, " \t\r\n", &tokptr);
     if (tok1 && tok2) {
       sscanf(tok1, "%x", &u);
       nameToUnicode->add(tok2, u);
@@ -788,7 +864,7 @@ GlobalParams::~GlobalParams() {
   deleteGooHash(unicodeMaps, GooString);
   deleteGooList(toUnicodeDirs, GooString);
   deleteGooHash(displayFonts, DisplayFontParam);
-#ifdef WIN32
+#ifdef _WIN32
   delete winFontList;
 #endif
   deleteGooHash(psFonts, PSFontParam);
@@ -926,6 +1002,7 @@ FILE *GlobalParams::findToUnicodeFile(GooString *name) {
   return NULL;
 }
 
+#if WITH_FONTCONFIGURATION_FONTCONFIG
 static GBool findModifier(const char *name, const char *modifier, const char **start)
 {
   const char *match;
@@ -944,7 +1021,6 @@ static GBool findModifier(const char *name, const char *modifier, const char **s
   }
 }
 
-#ifndef _MSC_VER
 static FcPattern *buildFcPattern(GfxFont *font)
 {
   int weight = -1,
@@ -1092,7 +1168,7 @@ static FcPattern *buildFcPattern(GfxFont *font)
 /* if you can't or don't want to use Fontconfig, you need to implement
    this function for your platform. For Windows, it's in GlobalParamsWin.cc
 */
-#ifndef _MSC_VER
+#if WITH_FONTCONFIGURATION_FONTCONFIG
 DisplayFontParam *GlobalParams::getDisplayFont(GfxFont *font) {
   DisplayFontParam *dfp;
   FcPattern *p=0;
@@ -1113,9 +1189,9 @@ DisplayFontParam *GlobalParams::getDisplayFont(GfxFont *font) {
 
     if (!p)
       goto fin;
-    FcConfigSubstitute(FCcfg, p, FcMatchPattern);
+    FcConfigSubstitute(NULL, p, FcMatchPattern);
     FcDefaultSubstitute(p);
-    set = FcFontSort(FCcfg, p, FcFalse, NULL, &res);
+    set = FcFontSort(NULL, p, FcFalse, NULL, &res);
     if (!set)
       goto fin;
     for (i = 0; i < set->nfont; ++i)
@@ -1152,6 +1228,9 @@ fin:
   return dfp;
 }
 #endif
+#if WITH_FONTCONFIGURATION_WIN32
+#include "GlobalParamsWin.cc"
+#endif
 
 GBool GlobalParams::getPSExpandSmaller() {
   GBool f;
@@ -1265,6 +1344,15 @@ GBool GlobalParams::getPSEmbedCIDTrueType() {
   return e;
 }
 
+GBool GlobalParams::getPSSubstFonts() {
+  GBool e;
+
+  lockGlobalParams;
+  e = psSubstFonts;
+  unlockGlobalParams;
+  return e;
+}
+
 GBool GlobalParams::getPSPreload() {
   GBool preload;
 
@@ -1541,11 +1629,11 @@ UnicodeMap *GlobalParams::getUnicodeMap2(GooString *encodingName) {
   return map;
 }
 
-CMap *GlobalParams::getCMap(GooString *collection, GooString *cMapName) {
+CMap *GlobalParams::getCMap(GooString *collection, GooString *cMapName, Stream *stream) {
   CMap *cMap;
 
   lockCMapCache;
-  cMap = cMapCache->getCMap(collection, cMapName);
+  cMap = cMapCache->getCMap(collection, cMapName, stream);
   unlockCMapCache;
   return cMap;
 }
@@ -1625,6 +1713,12 @@ void GlobalParams::setPSEmbedCIDTrueType(GBool embed) {
   unlockGlobalParams;
 }
 
+void GlobalParams::setPSSubstFonts(GBool substFonts) {
+  lockGlobalParams;
+  psSubstFonts = substFonts;
+  unlockGlobalParams;
+}
+
 void GlobalParams::setPSPreload(GBool preload) {
   lockGlobalParams;
   psPreload = preload;
